'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var tslib = require('tslib');
var template = require('@babel/template');

var metaPropNames = [
    'url',
    'filename',
    'dirname'
];
var isPropName = function (x) { return metaPropNames.includes(x); };
/**
 * Rewrites known `import.meta`[1] properties into equivalent non-module node.js
 * expressions. In order to maintain compatibility with plugins transforming
 * non-standard properties, this plugin transforms only known properties and
 * does not touch expressions with unknown or without member property access.
 * Properties known to this plugin:
 *
 * - `url`[2]
 *
 * [1]: https://github.com/tc39/proposal-import-meta
 * [2]: https://html.spec.whatwg.org/#hostgetimportmetaproperties
 */
var replacementForPropName = function (propName, target, esmUrlModuleId) {
    if (propName === 'filename') {
        return template.smart.ast(templateObject_1 || (templateObject_1 = tslib.__makeTemplateObject(["__filename"], ["__filename"])));
    }
    else if (propName === 'dirname') {
        return template.smart.ast(templateObject_2 || (templateObject_2 = tslib.__makeTemplateObject(["__dirname"], ["__dirname"])));
    }
    var urlModule = target === 'CommonJS' ? "require('url')" : esmUrlModuleId;
    return template.smart.ast(templateObject_3 || (templateObject_3 = tslib.__makeTemplateObject(["", ".pathToFileURL(__filename).toString()"], ["", ".pathToFileURL(__filename).toString()"])), urlModule);
};
function index () {
    return {
        name: 'transform-import-meta',
        visitor: {
            Program: function (path, state) {
                var _a;
                var _b = ((_a = state.opts) !== null && _a !== void 0 ? _a : {}).module, target = _b === void 0 ? 'CommonJS' : _b;
                if (target !== 'CommonJS' && target !== 'ES6') {
                    throw new Error('Invalid target, must be one of: "CommonJS" or "ES6"');
                }
                var metas = [];
                var identifiers = new Set();
                var hasUrl = false;
                path.traverse({
                    MemberExpression: function (memberExpPath) {
                        var node = memberExpPath.node;
                        if (node.object.type === 'MetaProperty' &&
                            node.object.meta.name === 'import' &&
                            node.object.property.name === 'meta' &&
                            node.property.type === 'Identifier' &&
                            isPropName(node.property.name)) {
                            if (node.property.name === 'url') {
                                hasUrl = true;
                            }
                            metas.push({ exp: memberExpPath, propName: node.property.name });
                            for (var _i = 0, _a = Object.keys(memberExpPath.scope.getAllBindings()); _i < _a.length; _i++) {
                                var name = _a[_i];
                                identifiers.add(name);
                            }
                        }
                    }
                });
                if (metas.length === 0) {
                    return;
                }
                var esmUrlModuleId = 'url';
                if (target === 'ES6') {
                    if (identifiers.has(esmUrlModuleId)) {
                        esmUrlModuleId = path.scope.generateUidIdentifier('url').name;
                    }
                    if (hasUrl) {
                        path.node.body.unshift(template.smart.ast(templateObject_4 || (templateObject_4 = tslib.__makeTemplateObject(["import ", " from 'url';"], ["import ", " from 'url';"])), esmUrlModuleId));
                    }
                }
                for (var _i = 0, metas_1 = metas; _i < metas_1.length; _i++) {
                    var meta = metas_1[_i];
                    var metaUrlReplacement = replacementForPropName(meta.propName, target, esmUrlModuleId);
                    meta.exp.replaceWith(metaUrlReplacement);
                }
            }
        }
    };
}
var templateObject_1, templateObject_2, templateObject_3, templateObject_4;

exports["default"] = index;
